# Generated by Django 4.0.4 on 2022-08-06 03:04

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=32)),
                ('last_name', models.CharField(max_length=32)),
                ('description', models.TextField()),
                ('favorite_number', models.IntegerField()),
                ('anniversary_datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('birth_date', models.DateField(default=django.utils.timezone.now)),
                ('weight_lbs', models.DecimalField(decimal_places=3, max_digits=7)),
            ],
        ),
    ]
